! ======================================================================
! Algebra lineal. Matriz real general
! ======================================================================

PROGRAM rge_lapack
IMPLICIT NONE

INTEGER i, j, k, info
INTEGER, PARAMETER:: n=3, la=n, ldl=n, ldr=n, lw=64*n, m=2, nsal=11
REAL(KIND(0.d0)):: a(la,la), b(la,m), wr(n), wi(n), vl(ldl,n), vr(ldr,n), &
                   work(lw), aa(la,la), bb(la,m)
CHARACTER jbl, jbr
INTEGER ipiv(n)

! Primera sentencia ejecutable
! ----------------------------

OPEN (nsal,FILE='RGE-lapack.sal')

aa = RESHAPE ( (/1d0,-1d0,0d0, -1d0,4d0,3d0, 2d0,-1d0,-1d0/),(/3,3/) )
! si aa(3,1) = 20/11 un autovalor es -2 y los otros 3(+|-)2i/SQRT(11)
aa(3,1) = 20d0/11d0

bb = RESHAPE ( (/2d0,2d0,2d0, 5d0,4d0,3d0/), (/3,2/))
WRITE (nsal,'(A,(T15,3F10.2))') 'Matriz a = ', (aa(i,:),i=1,n)
WRITE (nsal,'(/A,(T15,2F10.2))') 'RHS = ', (bb(i,:),i=1,n)

WRITE (nsal,'(/A/23("="))') 'Calculos con LAPACK'

! 1. Descomposicion LU. DGETRF (LAPACK)

a = aa
CALL dgetrf (n, n, a, la, ipiv, info)   ! a = LU, diag(L) = 1
WRITE (nsal,'(/A/31("-")/A,I2)') '1: Descomposicion LU (L(i,i)=1)',     &
       'info =', info
WRITE (nsal,'(/A,3I2)') 'Indices de pivotajes ipiv = ', ipiv(1:n)
DO i = 1, n
  WRITE (nsal,'(3F20.16)') a(i,1:n)
ENDDO

! 2. Sistema AX=B. DGETRS (LAPACK)

b = bb
CALL dgetrs ('N', n, m, a, la, ipiv, b, la, info)
WRITE (nsal,'(/A/15("-")/A,I2)') '2: Sistema AX=B', 'info =', info
WRITE (nsal,'(/A,(T15,2F20.16))') 'Solucion = ', (b(i,:),i=1,n)

! 3. Inversa. DGETRI (LAPACK)

CALL dgetri (n, a, la, ipiv, work, lw, info)
WRITE (nsal,'(/A/10("-")/A,I2)') '3: Inversa', 'info =', info
WRITE (nsal,'(/A,(T15,3F20.16))') 'Inversa = ', (a(i,:),i=1,n)

! 4. Autovalores y autovectores. DGEEV (LAPACK)

jbl = 'N' ; jbr = 'V'
a = aa
CALL dgeev (jbl, jbr, n, a, la, wr, wi, vl, ldl, vr, ldr, work, lw, info)

WRITE (nsal,'(/A/29("-")/A,I2)') '4: Autovalores y autovectores',       &
       'info = ', info
j = 1
DO
  IF (j > n) EXIT
  IF (wi(j) == 0.d0) THEN   ! autovalor real
    WRITE (nsal,'(/A,T15,F20.16)') 'Autovalor: ', wr(j)
    WRITE (nsal,'(A,T15,3F20.16)') 'Autovector: ', vr(1:n,j)
    j = j + 1
  ELSE   ! par de autovalores conjugados
    IF (j == n) STOP '*** ERROR. Esto no puede ocurrir ***'
    WRITE (nsal,'(/A,T15,F20.16,A,F20.16,A)') 'Autovalores: ',          &
                 wr(j),' +|-', wi(j),'i'
    WRITE (nsal,'(A,(T15,F20.16,A,F20.16,A))') 'Autovectores: ',        &
           (vr(k,j), ' +|-', vr(k,j+1),'i',k=1,n)
    j = j + 2
  ENDIF
ENDDO

ENDPROGRAM rge_lapack